package com.konaworks.idea.inspections.javaee.application;

import com.intellij.codeInspection.InspectionManager;
import com.intellij.codeInspection.LocalQuickFix;
import com.intellij.codeInspection.ProblemDescriptor;
import com.intellij.codeInspection.ProblemHighlightType;
import com.intellij.javaee.model.xml.application.JavaeeApplication;
import com.intellij.javaee.model.xml.application.JavaeeModule;
import com.intellij.psi.PsiElement;
import com.intellij.psi.util.PsiTreeUtil;
import com.intellij.psi.xml.XmlElement;
import com.intellij.psi.xml.XmlText;
import com.intellij.util.xml.DomFileElement;
import com.konaworks.idea.inspections.javaee.Bundle;
import com.konaworks.idea.inspections.javaee.DuplicateNamedElementsProcessor;
import org.jetbrains.annotations.NotNull;

import java.util.ArrayList;
import java.util.List;

public class AppDuplicatesInspection extends AbstractApplicationInspection {
  protected ProblemDescriptor[] checkDomFile(@NotNull DomFileElement<JavaeeApplication> domFileElement, @NotNull InspectionManager manager, boolean isOnTheFly) {
    final List<ProblemDescriptor> problems = new ArrayList<ProblemDescriptor>();

    checkContextRootsForDuplicates(domFileElement, problems, manager);

    checkWebUrisForDuplicates(domFileElement, problems, manager);
    
    return problems.toArray(new ProblemDescriptor[problems.size()]);

  }

  private void checkContextRootsForDuplicates(DomFileElement<JavaeeApplication> domFileElement, List<ProblemDescriptor> problems, InspectionManager manager) {
    final DuplicateNamedElementsProcessor<JavaeeModule> processor = new DuplicateNamedElementsProcessor<JavaeeModule>();

    for (JavaeeModule module : domFileElement.getRootElement().getModules()) {
      processor.put(module.getWeb().getContextRoot().getValue(), module);
    }

    for (JavaeeModule duplicate : processor.getDuplicates()) {
      final XmlElement xmlElement = duplicate.getWeb().getContextRoot().getXmlElement();
      if (xmlElement != null) {
        final PsiElement xmlText = PsiTreeUtil.getChildOfAnyType(xmlElement, XmlText.class);
        if (xmlText != null) {
          problems.add(manager.createProblemDescriptor(xmlText, Bundle.message("duplicate.context-root"), (LocalQuickFix) null, ProblemHighlightType.GENERIC_ERROR_OR_WARNING));
        }
      }
    }
  }

  private void checkWebUrisForDuplicates(DomFileElement<JavaeeApplication> domFileElement, List<ProblemDescriptor> problems, InspectionManager manager) {
    final DuplicateNamedElementsProcessor<JavaeeModule> processor = new DuplicateNamedElementsProcessor<JavaeeModule>();

    for (JavaeeModule module : domFileElement.getRootElement().getModules()) {
      processor.put(module.getWeb().getWebUri().getValue(), module);
    }

    for (JavaeeModule duplicate : processor.getDuplicates()) {
      final XmlElement xmlElement = duplicate.getWeb().getWebUri().getXmlElement();
      if (xmlElement != null) {
        final PsiElement xmlText = PsiTreeUtil.getChildOfAnyType(xmlElement, XmlText.class);
        if (xmlText != null) {
          problems.add(manager.createProblemDescriptor(xmlText, Bundle.message("duplicate.web-uri"), (LocalQuickFix) null, ProblemHighlightType.GENERIC_ERROR_OR_WARNING));
        }
      }
    }
  }

  @NotNull
  public String getDisplayName() {
    return Bundle.message("inspection.name.KW_AppDuplicatesInspection");
  }

  @NotNull
  public String getShortName() {
    return "KW_AppDuplicatesInspection";
  }
}
