package com.konaworks.idea.inspections.javaee.web;

import com.intellij.codeInsight.daemon.GroupNames;
import com.intellij.javaee.JavaeeDeploymentDescriptor;
import com.intellij.javaee.JavaeeModuleProperties;
import com.intellij.javaee.model.enums.WebAppVersion;
import com.intellij.javaee.model.xml.web.WebApp;
import com.intellij.openapi.module.Module;
import com.intellij.util.xml.DomElement;
import com.intellij.util.xml.highlighting.DomElementAnnotationHolder;
import com.intellij.util.xml.highlighting.DomElementsInspection;
import com.intellij.util.xml.highlighting.DomHighlightingHelper;
import org.jetbrains.annotations.Nls;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

public abstract class AbstractWebAppInspection extends DomElementsInspection<WebApp> {
  public AbstractWebAppInspection() {
    super(WebApp.class);
  }

  @Nls
  @NotNull
  public String getGroupDisplayName() {
    return GroupNames.JAVAEE_GROUP_NAME;
  }

  @Nullable
  protected static String getWebAppVersion(@NotNull WebApp webApp) {
    final WebAppVersion webAppVersion = webApp.getVersion().getValue();
    if (webAppVersion != null) {
      return webAppVersion.getValue();
    } else {
      final Module module = webApp.getModule();
      if (module != null) {
        final JavaeeModuleProperties javaeeModuleProperties = JavaeeModuleProperties.getInstance(module);
        if (javaeeModuleProperties != null) {
          final JavaeeDeploymentDescriptor javaeeDeploymentDescriptor = javaeeModuleProperties.getMainDeploymentDescriptor();
          if (javaeeDeploymentDescriptor != null) {
            return javaeeDeploymentDescriptor.getVersion();
          }
        }
      }
    }
    return null;
  }

  @Nullable
  protected static JavaeeDeploymentDescriptor[] getDeploymentDescriptors(@NotNull WebApp webApp) {
    final Module module = webApp.getModule();
    if (module != null) {
      final JavaeeModuleProperties javaeeModuleProperties = JavaeeModuleProperties.getInstance(module);
      if (javaeeModuleProperties != null) {
        return javaeeModuleProperties.getDeploymentDescriptors();
      }
    }
    return null;
  }

  protected void checkDomElement(DomElement element, DomElementAnnotationHolder holder, DomHighlightingHelper helper) {
    // todo rewrite subclasses to use this method. for now overwritten to prevent UnsupportedOperationException that breaks JBs generic inspections
  }
}
