package com.konaworks.idea.inspections.javaee.web;

import com.intellij.codeHighlighting.HighlightDisplayLevel;
import com.intellij.codeInspection.InspectionManager;
import com.intellij.codeInspection.LocalQuickFix;
import com.intellij.codeInspection.ProblemDescriptor;
import com.intellij.codeInspection.ProblemHighlightType;
import com.intellij.javaee.model.xml.EjbLocalRef;
import com.intellij.javaee.model.xml.EjbRef;
import com.intellij.javaee.model.xml.EnvEntry;
import com.intellij.javaee.model.xml.MessageDestination;
import com.intellij.javaee.model.xml.MessageDestinationRef;
import com.intellij.javaee.model.xml.ParamValue;
import com.intellij.javaee.model.xml.ResourceEnvRef;
import com.intellij.javaee.model.xml.ResourceRef;
import com.intellij.javaee.model.xml.web.ErrorPage;
import com.intellij.javaee.model.xml.web.Filter;
import com.intellij.javaee.model.xml.web.MimeMapping;
import com.intellij.javaee.model.xml.web.SecurityConstraint;
import com.intellij.javaee.model.xml.web.Servlet;
import com.intellij.javaee.model.xml.web.WebApp;
import com.intellij.javaee.model.xml.web.WebResourceCollection;
import com.intellij.openapi.project.Project;
import com.intellij.psi.PsiClass;
import com.intellij.psi.PsiElement;
import com.intellij.psi.util.PsiTreeUtil;
import com.intellij.psi.xml.XmlElement;
import com.intellij.psi.xml.XmlText;
import com.intellij.util.IncorrectOperationException;
import com.intellij.util.xml.DomFileElement;
import com.intellij.util.xml.GenericDomValue;
import com.konaworks.idea.inspections.javaee.Bundle;
import com.konaworks.idea.inspections.javaee.DuplicateNamedElementsProcessor;
import org.jetbrains.annotations.NonNls;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import javax.swing.JCheckBox;
import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import java.util.ArrayList;
import java.util.List;

public class WebAppExtraErrorDuplicatesInspection extends AbstractWebAppInspection {
  // duplicate names are checked within IDEA (since EAP 5581) for the elements:
  // filter/filter-name
  // servlet/servlet-name
  // security-role/role-name
  // resource-env-ref/resource-env-ref-name
  // resource-ref/res-ref-name
  // env-entry/env-entry-name
  // ejb-ref/ejb-ref-name
  // ejb-local-ref/ejb-ref-name
  // message-destination-ref/message-destination-ref-name

  private JPanel panel;

  private JCheckBox securityConstraintWebResourceNameCheckBox;
  private JCheckBox messageDestinationNameCheckBox;

  private JCheckBox filterClassCheckBox;
  private JCheckBox servletClassCheckBox;

  private JCheckBox filterInitParamCheckBox;
  private JCheckBox contextParamCheckBox;
  private JCheckBox servletInitParamCheckBox;

  private JCheckBox resourceNamesCheckBox;
  private JCheckBox errorPageCheckBox;
  private JCheckBox mimeMappingCheckBox;

  // settings readible and writable DefaultJDOMExternalizer
  public boolean checkSecurityConstraintWebResourceName = true;
  public boolean checkMessageDestinationName = true;

  public boolean checkFilterClass = true;
  public boolean checkServletClass = true;

  public boolean checkContextParam = true;
  public boolean checkFilterInitParam = true;
  public boolean checkServletInitParam = true;

  public boolean checkResourceNames = true;
  public boolean checkErrorPages = true;
  public boolean checkMimeMapping = true;

  public WebAppExtraErrorDuplicatesInspection() {
    securityConstraintWebResourceNameCheckBox.addChangeListener(new ChangeListener() {
      public void stateChanged(ChangeEvent e) {
        checkSecurityConstraintWebResourceName = securityConstraintWebResourceNameCheckBox.isSelected();
      }
    });
    messageDestinationNameCheckBox.addChangeListener(new ChangeListener() {
      public void stateChanged(ChangeEvent e) {
        checkMessageDestinationName = messageDestinationNameCheckBox.isSelected();
      }
    });

    filterClassCheckBox.addChangeListener(new ChangeListener() {
      public void stateChanged(ChangeEvent e) {
        checkFilterClass = filterClassCheckBox.isSelected();
      }
    });
    servletClassCheckBox.addChangeListener(new ChangeListener() {
      public void stateChanged(ChangeEvent e) {
        checkServletClass = servletClassCheckBox.isSelected();
      }
    });

    contextParamCheckBox.addChangeListener(new ChangeListener() {
      public void stateChanged(ChangeEvent e) {
        checkContextParam = contextParamCheckBox.isSelected();
      }
    });
    filterInitParamCheckBox.addChangeListener(new ChangeListener() {
      public void stateChanged(ChangeEvent e) {
        checkFilterInitParam = filterInitParamCheckBox.isSelected();
      }
    });
    servletInitParamCheckBox.addChangeListener(new ChangeListener() {
      public void stateChanged(ChangeEvent e) {
        checkServletInitParam = servletInitParamCheckBox.isSelected();
      }
    });

    resourceNamesCheckBox.addChangeListener(new ChangeListener() {
      public void stateChanged(ChangeEvent e) {
        checkResourceNames = resourceNamesCheckBox.isSelected();
      }
    });
    errorPageCheckBox.addChangeListener(new ChangeListener() {
      public void stateChanged(ChangeEvent e) {
        checkErrorPages = errorPageCheckBox.isSelected();
      }
    });
    mimeMappingCheckBox.addChangeListener(new ChangeListener() {
      public void stateChanged(ChangeEvent e) {
        checkMimeMapping = mimeMappingCheckBox.isSelected();
      }
    });
  }

  @NotNull
  public String getDisplayName() {
    return Bundle.message("inspection.name.KW_WebAppExtraErrorDuplicatesInspection");
  }

  @NotNull
  @NonNls
  public String getShortName() {
    return "KW_WebAppExtraErrorDuplicatesInspection";
  }

  @Nullable
  public JComponent createOptionsPanel() {
    securityConstraintWebResourceNameCheckBox.setSelected(checkSecurityConstraintWebResourceName);
    messageDestinationNameCheckBox.setSelected(checkMessageDestinationName);
    filterClassCheckBox.setSelected(checkFilterClass);
    servletClassCheckBox.setSelected(checkServletClass);
    contextParamCheckBox.setSelected(checkContextParam);
    filterInitParamCheckBox.setSelected(checkFilterInitParam);
    servletInitParamCheckBox.setSelected(checkServletInitParam);
    resourceNamesCheckBox.setSelected(checkResourceNames);
    errorPageCheckBox.setSelected(checkErrorPages);

    return panel;
  }

  @NotNull
  public HighlightDisplayLevel getDefaultLevel() {
    return HighlightDisplayLevel.ERROR;
  }

  protected ProblemDescriptor[] checkDomFile(@NotNull DomFileElement<WebApp> domFileElement, @NotNull InspectionManager manager, boolean isOnTheFly) {
    final List<ProblemDescriptor> problems = new ArrayList<ProblemDescriptor>();
    if (checkSecurityConstraintWebResourceName) { // in EAP 5581 these were not (yet) checked
      for (SecurityConstraint securityConstraint : domFileElement.getRootElement().getSecurityConstraints()) {
        final DuplicateNamedElementsProcessor<GenericDomValue<String>> processor = new DuplicateNamedElementsProcessor<GenericDomValue<String>>();
        for (WebResourceCollection webResourceCollection : securityConstraint.getWebResourceCollections()) {
          processor.put(webResourceCollection.getWebResourceName().getValue(), webResourceCollection.getWebResourceName());
        }
        for (GenericDomValue<String> webResName : processor.getDuplicates()) {
          final XmlElement xmlElement = webResName.getXmlElement();

          if (xmlElement != null) {
            final PsiElement xmlText = PsiTreeUtil.getChildOfAnyType(xmlElement, XmlText.class);
            if (xmlText != null) {
              problems.add(manager.createProblemDescriptor(xmlText, Bundle.message("duplicate.web-resource-name"), (LocalQuickFix) null, ProblemHighlightType.GENERIC_ERROR_OR_WARNING));
            }
          }
        }
      }
    }
    if (checkMessageDestinationName) { // in EAP 5581 these were not (yet) checked
      final DuplicateNamedElementsProcessor<GenericDomValue<String>> processor = new DuplicateNamedElementsProcessor<GenericDomValue<String>>();
      for (MessageDestination messageDestination : domFileElement.getRootElement().getMessageDestinations()) {
        processor.put(messageDestination.getMessageDestinationName().getValue(), messageDestination.getMessageDestinationName());
      }
      for (GenericDomValue<String> messageDestinationName : processor.getDuplicates()) {
        final XmlElement xmlElement = messageDestinationName.getXmlElement();

        if (xmlElement != null) {
          final PsiElement xmlText = PsiTreeUtil.getChildOfAnyType(xmlElement, XmlText.class);
          if (xmlText != null) {
            problems.add(manager.createProblemDescriptor(xmlText, Bundle.message("duplicate.message-destination-name"), (LocalQuickFix) null, ProblemHighlightType.GENERIC_ERROR_OR_WARNING));
          }
        }
      }
    }

    if (checkFilterClass) {
      final DuplicateNamedElementsProcessor<GenericDomValue<PsiClass>> processor = new DuplicateNamedElementsProcessor<GenericDomValue<PsiClass>>();
      for (Filter filter : domFileElement.getRootElement().getFilters()) {
        final PsiClass psiClass = filter.getFilterClass().getValue();
        if (psiClass != null) {
          processor.put(psiClass.getQualifiedName(), filter.getFilterClass());
        }
      }
      for (GenericDomValue<PsiClass> filterClass : processor.getDuplicates()) {
        final XmlElement xmlElement = filterClass.getXmlElement();
        if (xmlElement != null) {
          final PsiElement xmlText = PsiTreeUtil.getChildOfAnyType(xmlElement, XmlText.class);
          if (xmlText != null) {
            problems.add(manager.createProblemDescriptor(xmlText, Bundle.message("duplicate.filter-class"), (LocalQuickFix) null, ProblemHighlightType.GENERIC_ERROR_OR_WARNING));
          }
        }
      }
    }
    if (checkServletClass) {
      final DuplicateNamedElementsProcessor<GenericDomValue<PsiClass>> processor = new DuplicateNamedElementsProcessor<GenericDomValue<PsiClass>>();
      for (Servlet servlet : domFileElement.getRootElement().getServlets()) {
        final PsiClass psiClass = servlet.getServletClass().getValue();
        if (psiClass != null) {
          processor.put(psiClass.getQualifiedName(), servlet.getServletClass());
        }
      }
      for (GenericDomValue<PsiClass> servletClass : processor.getDuplicates()) {
        final XmlElement xmlElement = servletClass.getXmlElement();
        if (xmlElement != null) {
          final PsiElement xmlText = PsiTreeUtil.getChildOfAnyType(xmlElement, XmlText.class);
          if (xmlText != null) {
            problems.add(manager.createProblemDescriptor(xmlText, Bundle.message("duplicate.servlet-class"), (LocalQuickFix) null, ProblemHighlightType.GENERIC_ERROR_OR_WARNING));
          }
        }
      }
    }

    if (checkContextParam) {
      final DuplicateNamedElementsProcessor<GenericDomValue<String>> processor = new DuplicateNamedElementsProcessor<GenericDomValue<String>>();
      for (ParamValue contextParam : domFileElement.getRootElement().getContextParams()) {
        processor.put(contextParam.getParamName().getValue(), contextParam.getParamName());
      }
      for (GenericDomValue<String> paramName : processor.getDuplicates()) {
        final XmlElement xmlElement = paramName.getXmlElement();

        if (xmlElement != null) {
          final PsiElement xmlText = PsiTreeUtil.getChildOfAnyType(xmlElement, XmlText.class);
          if (xmlText != null) {
            problems.add(manager.createProblemDescriptor(xmlText, Bundle.message("duplicate.context-param.name"), DELETE_PARAM_FIX, ProblemHighlightType.GENERIC_ERROR_OR_WARNING));
          }
        }
      }

    }
    if (checkFilterInitParam) {
      for (Filter filter : domFileElement.getRootElement().getFilters()) {
        final DuplicateNamedElementsProcessor<GenericDomValue<String>> processor = new DuplicateNamedElementsProcessor<GenericDomValue<String>>();
        for (ParamValue initParam : filter.getInitParams()) {
          processor.put(initParam.getParamName().getValue(), initParam.getParamName());
        }

        for (GenericDomValue<String> paramName : processor.getDuplicates()) {
          final XmlElement xmlElement = paramName.getXmlElement();

          if (xmlElement != null) {
            final PsiElement xmlText = PsiTreeUtil.getChildOfAnyType(xmlElement, XmlText.class);
            if (xmlText != null) {
              problems.add(manager.createProblemDescriptor(xmlText, Bundle.message("duplicate.filter.init-param"), DELETE_PARAM_FIX, ProblemHighlightType.GENERIC_ERROR_OR_WARNING));
            }
          }
        }
      }
    }
    if (checkServletInitParam) {
      for (Servlet servlet : domFileElement.getRootElement().getServlets()) {
        final DuplicateNamedElementsProcessor<GenericDomValue<String>> processor = new DuplicateNamedElementsProcessor<GenericDomValue<String>>();
        for (ParamValue initParam : servlet.getInitParams()) {
          processor.put(initParam.getParamName().getValue(), initParam.getParamName());
        }

        for (GenericDomValue<String> paramName : processor.getDuplicates()) {
          final XmlElement xmlElement = paramName.getXmlElement();
          if (xmlElement != null) {
            final PsiElement xmlText = PsiTreeUtil.getChildOfAnyType(xmlElement, XmlText.class);
            if (xmlText != null) {
              problems.add(manager.createProblemDescriptor(xmlText, Bundle.message("duplicate.servlet.init-param"), DELETE_PARAM_FIX, ProblemHighlightType.GENERIC_ERROR_OR_WARNING));
            }
          }
        }
      }
    }

    if (checkResourceNames) {
      final DuplicateNamedElementsProcessor<GenericDomValue<String>> processor = new DuplicateNamedElementsProcessor<GenericDomValue<String>>();
      // some names defined are jndi names...
      for (EjbLocalRef ejbLocalRef : domFileElement.getRootElement().getEjbLocalRefs()) { // not using EnvironmentGroup IF
        processor.put(ejbLocalRef.getEjbRefName().getValue(), ejbLocalRef.getEjbRefName());
      }
      for (EjbRef ejbRef : domFileElement.getRootElement().getEjbRefs()) {
        processor.put(ejbRef.getEjbRefName().getValue(), ejbRef.getEjbRefName());
      }
      for (EnvEntry envEntry : domFileElement.getRootElement().getEnvEntries()) {
        processor.put(envEntry.getEnvEntryName().getValue(), envEntry.getEnvEntryName());
      }
      for (MessageDestinationRef messageDestinationRef : domFileElement.getRootElement().getMessageDestinationRefs()) {
        processor.put(messageDestinationRef.getMessageDestinationRefName().getValue(), messageDestinationRef.getMessageDestinationRefName());
      }
      for (ResourceEnvRef resourceEnvRef : domFileElement.getRootElement().getResourceEnvRefs()) {
        processor.put(resourceEnvRef.getResourceEnvRefName().getValue(), resourceEnvRef.getResourceEnvRefName());
      }
      for (ResourceRef resourceRef : domFileElement.getRootElement().getResourceRefs()) {
        processor.put(resourceRef.getResRefName().getValue(), resourceRef.getResRefName());
      }
      for (GenericDomValue<String> resourceName : processor.getDuplicates()) {
        final XmlElement xmlElement = resourceName.getXmlElement();
        if (xmlElement != null) {
          final PsiElement xmlText = PsiTreeUtil.getChildOfAnyType(xmlElement, XmlText.class);
          if (xmlText != null) {
            problems.add(manager.createProblemDescriptor(xmlText, Bundle.message("duplicate.resource.name"), DELETE_PARAM_FIX, ProblemHighlightType.GENERIC_ERROR_OR_WARNING));
          }
        }
      }
    }
    if (checkErrorPages) {
      final DuplicateNamedElementsProcessor<ErrorPage> processor = new DuplicateNamedElementsProcessor<ErrorPage>();

      for (ErrorPage errorPage : domFileElement.getRootElement().getErrorPages()) {
        processor.put(errorPage.getErrorCode().getStringValue(), errorPage);
      } // todo ? duplicate exception-type && exception-type specified in wrong order

      for (ErrorPage duplicate : processor.getDuplicates()) {
        final XmlElement errorPageXmlElement = duplicate.getErrorCode().getXmlElement();
        if (errorPageXmlElement != null) {
          final XmlText xmlText = PsiTreeUtil.getChildOfAnyType(errorPageXmlElement, XmlText.class);
          if (xmlText != null) {
            problems.add(manager.createProblemDescriptor(xmlText,
                Bundle.message("duplicate.error-page.error-code"), (LocalQuickFix) null,
                ProblemHighlightType.GENERIC_ERROR_OR_WARNING));
          }
        }
      }
    }
    if (checkMimeMapping) {
      final DuplicateNamedElementsProcessor<MimeMapping> processor = new DuplicateNamedElementsProcessor<MimeMapping>();

      for (MimeMapping mimeMapping : domFileElement.getRootElement().getMimeMappings()) {
        processor.put(mimeMapping.getExtension().getValue(), mimeMapping);
      }

      for (MimeMapping duplicate : processor.getDuplicates()) {
        final XmlElement extensionXmlElement = duplicate.getExtension().getXmlElement();
        if (extensionXmlElement != null) {
          final XmlText xmlText = PsiTreeUtil.getChildOfAnyType(extensionXmlElement, XmlText.class);
          if (xmlText != null) {
            problems.add(manager.createProblemDescriptor(xmlText,
                Bundle.message("duplicate.mime-mapping.extension"), (LocalQuickFix) null,
                ProblemHighlightType.GENERIC_ERROR_OR_WARNING));
          }
        }
      }
    }

    return problems.toArray(new ProblemDescriptor[problems.size()]);
  }

  private static final DeleteQuickFix DELETE_PARAM_FIX = new DeleteQuickFix();

  private static class DeleteQuickFix implements LocalQuickFix {
    @NotNull
    public String getName() {
      return Bundle.message("quickfix.remove.duplicate.param");
    }

    public void applyFix(@NotNull Project project, ProblemDescriptor descriptor) {
      try {
        descriptor.getPsiElement().getParent()/*xxx-name*/.getParent().delete();
      }
      catch (IncorrectOperationException e) {
        // do nothing
      }
    }

    @NotNull
    public String getFamilyName() {
      return getName();
    }
  }
}
