package com.konaworks.idea.intentions.javaee.web;

import com.intellij.codeInsight.intention.IntentionAction;
import com.intellij.javaee.model.xml.web.WebApp;
import com.intellij.openapi.editor.Editor;
import com.intellij.openapi.project.Project;
import com.intellij.psi.PsiFile;
import com.intellij.psi.xml.XmlFile;
import com.intellij.util.IncorrectOperationException;
import com.intellij.util.xml.DomFileElement;
import com.intellij.util.xml.DomManager;
import org.jetbrains.annotations.NonNls;
import org.jetbrains.annotations.NotNull;

public abstract class AbstractWebAppIntention implements IntentionAction {
  @NonNls
  private static final String WEB_XML = "web.xml";

  @NotNull
  public final String getFamilyName() {
    return "Konaworks EE Intentions";
  }

  public final boolean isAvailable(Project project, Editor editor, PsiFile file) {
    if (WEB_XML.equals(file.getName())) {
      final DomFileElement<WebApp> domFileElement = DomManager.getDomManager(file.getProject()).getFileElement((XmlFile) file, WebApp.class);

      final WebApp webApp;
      if (domFileElement != null) {
        webApp = domFileElement.getRootElement();
        return isAvailable(project, editor, webApp);
      }
    }
    return false;
  }

  public final void invoke(Project project, Editor editor, PsiFile file) throws IncorrectOperationException {
    final DomFileElement<WebApp> domFileElement = DomManager.getDomManager(project).getFileElement((XmlFile) file, WebApp.class);

    if (domFileElement != null) {
      final WebApp webApp = domFileElement.getRootElement();
      invoke(project, editor, webApp);
    }
  }

  public abstract boolean isAvailable(Project project, Editor editor, WebApp webApp);

  public abstract void invoke(Project project, Editor editor, WebApp webApp);
}
