package com.konaworks.idea.intentions.javaee.web;

import com.intellij.javaee.model.xml.Description;
import com.intellij.javaee.model.xml.SecurityRole;
import com.intellij.javaee.model.xml.web.WebApp;
import com.intellij.openapi.editor.Editor;
import com.intellij.openapi.project.Project;
import com.intellij.psi.PsiElement;
import com.intellij.psi.PsiFile;
import com.intellij.psi.util.PsiTreeUtil;
import com.intellij.psi.xml.XmlTag;
import com.intellij.util.IncorrectOperationException;
import com.intellij.util.xml.DomElement;
import com.intellij.util.xml.DomManager;
import org.jetbrains.annotations.NotNull;

import java.util.ArrayList;
import java.util.List;
import java.util.SortedMap;
import java.util.TreeMap;

public class WebAppSortedSecurityRolesIntention extends AbstractWebAppIntention {
  public boolean isAvailable(Project project, Editor editor, WebApp webApp) {
    final PsiFile psiFile = webApp.getRoot().getFile();

    PsiElement elementAtPointer = psiFile.findElementAt(editor.getCaretModel().getOffset());

    if (elementAtPointer == null) {
      return false;
    }

    if (webApp.getSecurityRoles().size() < 2) {
      return false;
    }

    if ((!(elementAtPointer instanceof XmlTag))) {

      elementAtPointer = PsiTreeUtil.getParentOfType(elementAtPointer, XmlTag.class);
      if (elementAtPointer == null) {
        return false;
      }
    }

    DomElement domElement = DomManager.getDomManager(project).getDomElement((XmlTag) elementAtPointer);

    while (domElement != null && (!(domElement instanceof SecurityRole))) {
      domElement = domElement.getParent();
    }

    return domElement != null && domElement.getParent() instanceof WebApp;
  }

  public void invoke(Project project, Editor editor, WebApp webApp) {
    // todo bug : xml comments get lost. probably rework to xml based.
    final SortedMap<String, DummySecurityRole> tempRoles = new TreeMap<String, DummySecurityRole>();

    for (SecurityRole role : webApp.getSecurityRoles()) {
      tempRoles.put(role.getRoleName().getValue(), new DummySecurityRole(role));
      try {
        //noinspection ConstantConditions
        role.getXmlElement().delete();
      } catch (IncorrectOperationException e) {
        e.printStackTrace(); // todo
      }
    }

    for (DummySecurityRole dummy : tempRoles.values()) {
      final SecurityRole role = webApp.addSecurityRole();
      role.getRoleName().setValue(dummy.getRoleName());

      for (String descriptionValue : dummy.getDescriptions()) {
        final Description description = role.addDescription();
        description.setValue(descriptionValue);
      }
    }
  }

  @NotNull
  public String getText() {
    return "Sort Security Roles";
  }

  public boolean startInWriteAction() {
    return true;
  }

  private class DummySecurityRole {
    private String roleName;
    private List<String> descriptions = new ArrayList<String>();

    public DummySecurityRole(SecurityRole role) {
      roleName = role.getRoleName().getValue();

      for (Description description : role.getDescriptions()) {
        descriptions.add(description.getValue());
      }
    }

    public String getRoleName() {
      return roleName;
    }

    public List<String> getDescriptions() {
      return descriptions;
    }
  }
}
